﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data.Entity;
using System.Threading.Tasks;
using System.Windows.Data;
using System.Windows.Input;

namespace GUI
{
    class MainViewModel : IDisposable, INotifyPropertyChanged
    {
        /*************************************
         * Attributes
         * **********************************/
        private DatabaseContainer db = null;
        public event PropertyChangedEventHandler PropertyChanged;

        public CollectionViewSource _feedItemSource;
        public CollectionViewSource FeedItemSource
        {
            get
            {
                if (_feedItemSource == null)
                    GetMasterEntityCollection();
                return _feedItemSource;
            }
        }

        private FeedItem _currentFeedItem;
        public FeedItem CurrentFeedItem
        {
            get
            {
                return _currentFeedItem;
            }
            set
            {
                _currentFeedItem = value;
                NotifyPropertChanged("CurrentFeedItem");
            }
        }

        private CollectionViewSource _feedSource;
        public CollectionViewSource FeedSource
        {
            get
            {
                if (_feedSource == null)
                    GetMasterEntityCollection();
                return _feedSource;
            }
        }

        /*************************************
         * Commands
         * **********************************/
        private ICommand _deleteSelectedFeed;
        public ICommand DeleteSelectedFeed
        {
            get
            {
                if (_deleteSelectedFeed == null)
                    _deleteSelectedFeed = new DelegateCommand((x) => DeleteFeed());
                return _deleteSelectedFeed;
            }
        }

        private ICommand _refreshFeeds;
        public ICommand RefreshFeeds
        {
            get
            {
                if (_refreshFeeds == null)
                    _refreshFeeds = new DelegateCommand((x) => UpdateAllFeedsAsync());
                return _refreshFeeds;
            }
        }

        private ICommand _addFeed;
        public ICommand AddFeed
        {
            get
            {
                if (_addFeed == null)
                    _addFeed = new DelegateCommand((x) => CreateFeed(x));
                return _addFeed;
            }
        }

        /*************************************
         * Methods
         * **********************************/
        public MainViewModel()
        {
            db = new DatabaseContainer();
        }

        public void Dispose()
        {
            db.Dispose();
        }

        public void NotifyPropertChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        private void GetMasterEntityCollection()
        {
            _feedItemSource = new CollectionViewSource();
            _feedSource = new CollectionViewSource();

            db.FeedSet.Load();
            _feedSource.Source = db.FeedSet.Local;
            _feedSource.View.CurrentChanged += (x, y) =>
            {
                Feed selectedFeed = _feedSource.View.CurrentItem as Feed;
                if (selectedFeed != null)
                {
                    _feedItemSource.Source = selectedFeed.FeedItem;
                    SetCurrentFeedItem();
                    
                    if (_feedItemSource.View != null)
                    {
                        _feedItemSource.View.CurrentChanged += (a, b) =>
                        {
                            FeedItem selectedFeedItem = _feedItemSource.View.CurrentItem as FeedItem;
                            if (selectedFeedItem != null)
                            {
                                SetCurrentFeedItem();
                            }
                        };
                    };
                }
            };
            _feedSource.View.Refresh();
        }

        private void SetCurrentFeedItem()
        {
            CurrentFeedItem = _feedItemSource.View.CurrentItem as FeedItem;
            if (CurrentFeedItem != null)
            {
                MarkRead(CurrentFeedItem);
            }
        }

        private void CreateFeed(object url)
        {
            String sUrl = url as string;
            if (sUrl != null)
            {
                CreateFeed(sUrl);
            }
        }

        private void CreateFeed(String url)
        {
            Feed feed = FeedHandler.GetFeedInformation(url);
            db.FeedSet.Add(feed);
            db.SaveChanges();
        }

        private void RemoveFeed(Feed feed)
        {
            db.FeedItemSet.RemoveRange(feed.FeedItem);
            db.FeedSet.Remove(feed);
            db.SaveChanges();
        }

        private async void UpdateAllFeedsAsync()
        {
            List<Task> tasks = new List<Task>();
            foreach (Feed feed in db.FeedSet)
            {
                feed.Updating = true;
                Task task = FeedHandler.UpdateFeedAsync(feed).ContinueWith(x =>
                {
                    feed.Updating = false;
                    feed.NotifyPropertChanged("Unread");
                });
                tasks.Add(task);
            }
            await Task.WhenAll(tasks.ToArray());
            _feedItemSource.View.Refresh();
            db.SaveChanges();
        }

        private void MarkRead(FeedItem fi)
        {
            fi.Read = true;
            fi.NotifyPropertChanged("Read");
            fi.Feed.NotifyPropertChanged("Unread");
            db.SaveChanges();
        }

        private void DeleteFeed()
        {
            Feed selectedFeed = _feedSource.View.CurrentItem as Feed;
            if (selectedFeed != null)
            {
                RemoveFeed(selectedFeed);
            }
        }
    }
}

